%!octave 
%
%     KNP
%      
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.6.4
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: knp.m,v 1.4 2014/02/08 10:55:29 mechanoid Exp $


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function retval = knp(R)
   szX = size(R,1) ;

   % матрица инцидентности графа 
   G = zeros(szX);
   
   if szX<2, retval=G; return; endif

   % для удобства поиска минимумов в матрице расстояний R
   % присваиваем главной диагонали R большие значения 
   MAX_DISTANCE = 10e7 ;
   E = eye(szX).*MAX_DISTANCE ;
   R+=E;

   
   % найти пару точек с номерами i,j 
   % и с наименьшим расстоянием R 
   [ v i ] = min(R) ; [ v j ] = min(v) ; i = i(j);
  
   % и соединить  i,j ребром
   G(i,j) = 1 ;
   G(j,i) = 1 ;

   % список номеров связанных точек
   assoc = [ i j ] ;

   % список номеров изолированных точек
   free = 1:szX ;
   % удалить из списка номеров изолированных точек
   % две точки связанные на предыдущем шаге
   if i > j, free(i) = [] ; free(j) = [] ; else free(j) = [] ; free(i) = [] ; endif
   szFree = szX-2 ;

   % пока в выборке остаются изолированные точки
   while szFree>1,
	  % найти пару точек 
	  % изолированную номер i и связанную номер j 
	  % с наименьшим расстоянием R
	  [ v i ] = min( R( free , assoc ) );
	  [ v j ] = min( v );

	  fi = i(j) ;
	  % номер изолированной точки
	  i = free(fi) ; 
	  % номер связанной точки
	  j = assoc(j) ; 

	  % соединить точки номер j и i ребром
	  G(i,j) = 1 ;
      G(j,i) = 1 ;
	  
	  % скорректировать списки связанных и изолированных точек
	  assoc = [ assoc i j ] ;
	  free(fi) = []; szFree -= 1 ;

	  printf("%i    \r",szFree);
   endwhile
   %printf("\n");

   % присоединить последнюю точку
   [ v j ] = min( R( free , assoc ) );
   i = free ; j = assoc(j) ; 
   G(i,j) = 1 ;
   G(j,i) = 1 ;

   save('-ascii','result/G.txt','G');

   retval = G;
endfunction


