%!octave 
%
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.6.4
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: quality_clusters.m,v 1.1 2014/02/09 11:10:46 mechanoid Exp $

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval = quality_clusters(R,S),
   % количество кластеров 
   szS=size(S,2) ;

   res=[] ;
   for i=1:szS,
	  printf('%i/%i       \r',i,szS);
	  C=S{i} ;
      % обрабатываем кластер номер i
	  r = sum_distances_cluster(R,C) ;
	  res=[ res ; r ] ;
   endfor
   % результаты для всех кластеров 
   res=sum(res) ;

   % среднее внутрикластерное расстояние и среднее межкластерное расстояние
   mn=[ (res(1)/res(2))  (res(3)/res(4)) ] ;

   % качество кластеризации 
   retval = szS * mn(1)/mn(2) ;

endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=sum_distances_cluster(R,C), 
   % сумма внутренних расстояний для кластера С
   [ di ci ] = sum_distances_in(R,C) ;

   % сумма внешних расстояний для кластера С
   [ dm cm ] = sum_distances_out(R,C) ;
   
   % матрица результатов для кластера С
   retval=[ di ci dm cm ] ;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% сумма расстояний между точками из С
function [ d i ] = sum_distances_in(R,C),
   szC=size(C,2) ;
   if szC<2, d=0; i=1;return; endif
   R=triu(R,1) ;
   d=sum(sum(R(C,C))) ;
   % количество пар (сочетаний по 2) из szC  
   i=nchoosek(szC,2) ;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% сумма расстояний между точками из C и точками X\C
function [ d i ] = sum_distances_out(R,C),
   X=1:size(R,1) ;
   Xo=setdiff(X,C) ;
   d=sum(sum(R(Xo,C))) ;
   i=size(C,2)*size(Xo,2) ;
endfunction





% среднее внутрикластерное расстояние -> min
% 
%    сумма расстояний от каждой точки набора до всех точек её кластера
%    /
%    общее количество обработанных точек
% 
% 
% среднее межкластерное расстояние -> max
% 
%    сумма расстояний от каждой точки набора до всех точек не принадлежащих её кластеру
%    /
%    общее количество обработанных точек
% 
% 
% качество кластеризации -> min
% 
%    количество кластеров
%    *
%    среднее внутрикластерное расстояние
%    /
%    среднее межкластерное расстояние


